﻿package com.ipernity.api
{
	/* ============================================================================
	// IperAPI.as
	// A pure actionscript toolkit for ipernity API.
	// @version: 0.9
	// @release-date: 2008-10-10
	// @author: Franck Priola
	//
	// ---------------------------------------------------------------------------- */
	
	/* ****************************************************************************
	 * PLEASE BEHAVE LIKE A NICE AND MATURE PROGRAMMER
	 * - do not make hundred of API call per second
	 * - do not make unecessary calls when you can cache results
	 * **************************************************************************** */

	 
	import flash.events.Event;
	import flash.events.SecurityErrorEvent;
	import flash.events.IOErrorEvent;
	
	import com.adobe.serialization.json.JSON;
	import com.adobe.crypto.MD5;
	
	import com.ipernity.api.IperAPILoader;
	import com.ipernity.api.IperAPIUtils;
	
	
	public class IperAPI
	{
		/**
		 * default configuration
		 *     api_key (String)      : (needed)   your API key 
		 *     api_secret (String)   : (optional) your API secret [* see warning below]
		 *     format (String)       : (needed)   "json/xml" - response format
		 *     raw_response (String) : (needed)   "yes/no" - raw data response (String for json/xml) or Object (for json format) / XML (for xml format)
		 *
		 *  Warning! If you put the "API secret" in your Flash code, please use a code obfuscator (in fact,
		 *           the API secret is only necessary with some API functions, check if you really need it).
		 *           Otherwise, your code could be decompiled and your secret stolen.
		 *           For security reasons, instead of using the secret key and the API key, use the PHP proxy provided
		 *           in the Javascript API kit (if you do this, remember to change api_url below).
		 *                 
		 */
		private var _conf:Object = {
              api_key      : ""
             ,api_secret   : ""
             ,format       : "xml"
			 ,raw_response : false
             ,api_url      : "http://api.ipernity.com/api"                 // API url
             ,auth_url     : "http://www.ipernity.com/apps/authorize"      // authentification url
			 ,version      : "0.9"
		}
		
		/** for authentication */
		private var _auth:Object = {
			"token":"", "user_id":0, "username":"", "realname":"", "permissions":null
		};
		
			
		/**
		 * IperAPI()
		 * 		Constructor
		 */
		public function IperAPI( conf:Object ):void
		{
			_setConf( conf );
		}
		
		
		/**
		 * request()
		 * 		Perform an API call with method and parameters
		 */
		public function request( method:String, params:Object, callback:Function, withAuthToken:Boolean = false ):void
		{
			/** add api key */
			params["api_key"] = _conf["api_key"];
			
			/** add auth_token ? */
			if( withAuthToken && _auth["token"] ) {
			  params["auth_token"] = _auth["token"];
			}
			
			/** sign request */
			if( _conf["api_secret"] ) {
				params["api_sig"] = _signParams( method, params );
			}
	
			/** make query url */
			var url:String = _conf['api_url'] + '/' + method + '/' + _conf["format"];
			
			/** instanciate the dynamic API loader */
			var loader:IperAPILoader = new IperAPILoader();
			loader.method = method;
			loader.url = url;
			loader.params = params;
			loader.callback = callback;
			loader.format = _conf["format"];
			loader.raw_response = _conf["raw_response"];
			
			/** listen API loader events */
			loader.addEventListener( Event.COMPLETE, _onComplete );
            loader.addEventListener( SecurityErrorEvent.SECURITY_ERROR, _onError );
			loader.addEventListener( IOErrorEvent.IO_ERROR, _onError );
			
			/** send request */
			loader.send();
		}

		
		/**
		 * request complete
		 */
        private function _onComplete( e:Event ):void
		{
			var loader:Object = e.target;
			var response:*;
			var success:Boolean;
			
			
			/** check JSON response */
			if( loader.format == "json" )
			{
				try {
					var j:Object = JSON.decode( loader.data );
				}
				catch( err:Error ) {
					sendError( loader, "Incorrect JSON data." );
					return;
				}
				
				if( loader.raw_response ) {
					response = loader.data;
				} else {
					response = j;
				}
				
				if ( !j["api"] || !j["api"]["status"] )
				{
					sendError( loader, "Could not parse JSON response." );
					return;
				}
				else
				{
					success = j["api"]["status"] == "ok" ? true : false;
				}
			}
		
			/** check XML response */
			else if( loader.format == "xml" )
			{
				try {
					var x:XML = new XML( loader.data );
				}
				catch( err:Error ) {
					sendError( loader, "Incorrect XML data." );
					return;
				}
				
				if( loader.raw_response ) {
					response = loader.data
				} else {
					response = x;
				}
				
				if( !x.attribute("status") )
				{
					sendError( loader, "Could not parse XML response." );
					return;
				}
				else
				{
					success = x.attribute("status") == "ok" ? true : false;
				}
			}
		
			/** bad output format */
			else
			{
				sendError( loader, "Invalid API output format : '"+loader.format+"'" );
				return;
			}
			
			
			/** execute the callback method */
			loader.callback( success, response, loader );
        }

		/**
		 * request error
		 */
		private function _onError( e:* ):void
		{
			sendError( e.target, e.text );
        }

		
		/**
		 * sendError()
		 *		if an error has occured (not API), build the error response
		 */
		private function sendError( loader:Object, errorMessage:String ):void
		{
			var response:*;
			
			/** default message */
			if( !errorMessage ) errorMessage = "API call failed. This may be a connection problem. Please try again later.";
			
			/** make the error response */
			if( loader.format == "json" )
			{
				if( loader.raw_response )
				{
					response = '{"api":{"status":"error","code":0,"message":"'+errorMessage+'"}}';
				}
				else
				{
					response = { "status":"error", "code":0, "message":errorMessage };
				}
			}
			else if( loader.format == "xml" )
			{
				if( loader.raw_response )
				{
					response = '<api status="error" code="0" message="'+errorMessage+'"/>';
				}
				else
				{
					response = new XML( '<api status="error" code="0" message="'+errorMessage+'"/>' );
				}
			}
			
			/** execute the callback method */
			loader.callback( false, response, loader );
		}

		
		/**
		 * _setConf()
		 * 		Update default conf
		 */
		private function _setConf( conf:Object ):void
		{
			for( var i:String in conf ) {
            	_conf[i] = conf[i];
        	}
		}
		
		
		/**
		 * _signParams()
		 * 		return the parameters api_sig value
		 */
		private function _signParams( method:String, params:Object ):String
		{
			var keys:Array = IperAPIUtils.getAllKeys( params, true );
		  
			var sig:String = "";
			for( var i = 0; i < keys.length; i++ ) { sig += keys[i] + params[keys[i]]; }
			sig += method + _conf["api_secret"];
			
			return MD5.hash( sig );
		}
		
		
		/**
		 * setAuthToken()
		 * 		save the authentication token
		 */
		public function setAuthToken( auth_token:String ):void
		{
			_auth["token"] = auth_token;
		}
	}
}
